/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.exml.workspace.api.node.customizer.BasicRenderingInformation;
import ro.sync.exml.workspace.api.node.customizer.NodeRendererCustomizerContext;
import ro.sync.exml.workspace.api.node.customizer.XMLNodeRendererCustomizer;

/**
 * Class used to customize the way a DITA node is rendered in the UI. 
 * A node represents an entry from Author outline, Author bread crumb,
 * Text page outline, content completion proposals window or Elements view.
 * 
 * @author alin_balasa
 * @author alina_iordache
 */
@API(type = APIType.INTERNAL, src = SourceType.PUBLIC)
public class DITANodeRendererCustomizer extends XMLNodeRendererCustomizer {
  
  /**
   * Logger for logging.
   */
  private static final Logger logger = Logger.getLogger(DITANodeRendererCustomizer.class.getName());
  
  /**
   * DITA classes constants.
   */
  public static enum DitaClass {
    /**
     * Title class.
     */
    TITLE_CLASS(" topic/title "),

    /**
     * Topic body class.
     */
    TOPIC_BODY_CLASS(" topic/body "),

    /**
     * Concept body class.
     */
    CONCEPT_BODY_CLASS(" concept/conbody "),

    /**
     * Task body class.
     */
    TASK_BODY_CLASS(" task/taskbody "),

    /**
     * Paragraph class.
     */
    P_CLASS(" topic/p "),

    /**
     * Italic class.
     */
    I_CLASS(" hi-d/i "),

    /**
     * Bold class.
     */
    B_CLASS(" hi-d/b "),

    /**
     * Underline class.
     */
    U_CLASS(" hi-d/u "),

    /**
     * Image class.
     */
    IMAGE_CLASS(" topic/image "),

    /**
     * Link class.
     */
    LINK_CLASS(" topic/link "),

    /**
     * XREF class.
     */
    XREF_CLASS(" topic/xref "),

    /**
     * Table class.
     */
    TABLE_CLASS(" topic/table "),

    /**
     * Simple table class.
     */
    SIMPLE_TABLE_CLASS(" topic/simpletable "),

    /**
     * THead class.
     */
    THEAD_CLASS(" topic/thead "),

    /**
     * STHead class.
     */
    STHEAD_CLASS(" topic/sthead "),

    /**
     * TBody class.
     */
    TBODY_CLASS(" topic/tbody "),

    /**
     * Row class.
     */
    ROW_CLASS(" topic/row "),

    /**
     * STRow class.
     */
    STROW_CLASS(" topic/strow "),

    /**
     * Entry class.
     */
    ENTRY_CLASS(" topic/entry "),

    /**
     * STEntry class.
     */
    STENTRY_CLASS(" topic/stentry "),

    /**
     * UL class.
     */
    UL_CLASS(" topic/ul "),

    /**
     * OL class.
     */
    OL_CLASS(" topic/ol "),

    /**
     * Steps class.
     */
    STEPS_CLASS(" task/steps "),

    /**
     * Step class.
     */
    STEP_CLASS(" task/step "),

    /**
     * LI class.
     */
    LI_CLASS(" topic/li "),

    /**
     * Body div class.
     */
    BODY_DIV_CLASS(" topic/bodydiv "),
    
    /**
     * Section class.
     */
    SECTION_CLASS(" topic/section "),
    
    /**
     * Topic class.
     */
    TOPIC_CLASS(" topic/topic "),
    
    /**
     * Map class.
     */
    MAP_CLASS(" map/map "),

    /**
     * Bookmap class.
     */
    BOOKMAP_CLASS(" bookmap/bookmap "),
    
    /**
     * Task class.
     */
    TASK_CLASS(" task/task "),
    
    /**
     * Concept class.
     */
    CONCEPT_CLASS(" concept/concept "),
    
    /**
     * Index term class.
     */
    INDEXTERM_CLASS(" topic/indexterm "),
    
    /**
     * Object class.
     */
    OBJECT_CLASS(" topic/object ");

    /**
     * The class value.
     */
    private String value; 

    /**
     * Constructor
     * 
     * @param value The class value.
     */
    DitaClass(String value) {
      this.value = value;
    }
    
    /**
     * @return Returns the value.
     */
    public String getValue() {
      return value;
    }
  }
  
  /**
   * Mapping between name and icon path
   */
  private static final Map<DitaClass, String> classToIconPath = new HashMap<DitaClass, String>();

  // Initialize mapping
  static {
    // title
    classToIconPath.put(DitaClass.TITLE_CLASS, getImageURL("/images/node-customizer/ElementTitle16.png"));

    String bodyImg = getImageURL("/images/node-customizer/ElementBody16.png");

    // body
    classToIconPath.put(DitaClass.TOPIC_BODY_CLASS, bodyImg);

    // taskbody
    classToIconPath.put(DitaClass.TASK_BODY_CLASS, bodyImg);

    // conbody
    classToIconPath.put(DitaClass.CONCEPT_BODY_CLASS, bodyImg);

    // paragraph
    classToIconPath.put(DitaClass.P_CLASS, getImageURL("/images/node-customizer/ElementPara16.png"));

    // italic
    classToIconPath.put(DitaClass.I_CLASS, getImageURL("/images/node-customizer/ElementItalic16.png"));

    // bold
    classToIconPath.put(DitaClass.B_CLASS, getImageURL("/images/node-customizer/ElementBold16.png"));

    // underline
    classToIconPath.put(DitaClass.U_CLASS, getImageURL("/images/node-customizer/ElementUnderline16.png"));

    // image
    classToIconPath.put(DitaClass.IMAGE_CLASS, getImageURL("/images/node-customizer/ElementImage16.png"));

    // xref
    classToIconPath.put(DitaClass.XREF_CLASS, getImageURL("/images/node-customizer/ElementXref16.png"));

    // link
    classToIconPath.put(DitaClass.LINK_CLASS, getImageURL("/images/node-customizer/ElementLink16.png"));

    String tableImg = getImageURL("/images/node-customizer/ElementTable16.png");

    // table
    classToIconPath.put(DitaClass.TABLE_CLASS, tableImg);

    // simpletable
    classToIconPath.put(DitaClass.SIMPLE_TABLE_CLASS, tableImg);

    String tableHeadImg = getImageURL("/images/node-customizer/ElementTHead16.png");

    // table head
    classToIconPath.put(DitaClass.THEAD_CLASS, tableHeadImg);

    // table head
    classToIconPath.put(DitaClass.STHEAD_CLASS, tableHeadImg);

    // table body
    classToIconPath.put(DitaClass.TBODY_CLASS, getImageURL("/images/node-customizer/ElementTBody16.png"));

    String tableRowImg = getImageURL("/images/node-customizer/ElementTr16.png");

    // table row
    classToIconPath.put(DitaClass.ROW_CLASS, tableRowImg);

    // table row
    classToIconPath.put(DitaClass.STROW_CLASS, tableRowImg);

    String tableCellImg = getImageURL("/images/node-customizer/ElementTd16.png");

    // table cell
    classToIconPath.put(DitaClass.ENTRY_CLASS, tableCellImg);

    // table cell
    classToIconPath.put(DitaClass.STENTRY_CLASS, tableCellImg);

    // unordered list
    classToIconPath.put(DitaClass.UL_CLASS, getImageURL("/images/node-customizer/ElementUl16.png"));

    String olImg = getImageURL("/images/node-customizer/ElementOl16.png");

    // ordered list
    classToIconPath.put(DitaClass.OL_CLASS, olImg);

    // steps
    classToIconPath.put(DitaClass.STEPS_CLASS, olImg);

    String liImg = getImageURL("/images/node-customizer/ElementLi16.png");

    // step
    classToIconPath.put(DitaClass.STEP_CLASS, liImg);

    // li
    classToIconPath.put(DitaClass.LI_CLASS, liImg);

    // bodydiv
    classToIconPath.put(DitaClass.BODY_DIV_CLASS, getImageURL("/images/node-customizer/ElementDiv16.png"));

    // section
    classToIconPath.put(DitaClass.SECTION_CLASS, getImageURL("/images/node-customizer/ElementSection16.png"));

    // topic
    classToIconPath.put(DitaClass.TOPIC_CLASS, getImageURL("/images/node-customizer/ElementTopic16.png"));

    // map
    classToIconPath.put(DitaClass.MAP_CLASS, getImageURL("/images/node-customizer/ElementMap16.png"));

    // bookmap
    classToIconPath.put(DitaClass.BOOKMAP_CLASS, getImageURL("/images/node-customizer/ElementBookmap16.png"));

    // task
    classToIconPath.put(DitaClass.TASK_CLASS, getImageURL("/images/node-customizer/ElementTask16.png"));

    // concept
    classToIconPath.put(DitaClass.CONCEPT_CLASS, getImageURL("/images/node-customizer/ElementConcept16.png"));

    // indexterm
    classToIconPath.put(DitaClass.INDEXTERM_CLASS, getImageURL("/images/node-customizer/ElementIndexterm16.png"));
    
    // object
    classToIconPath.put(DitaClass.OBJECT_CLASS, getImageURL("/images/node-customizer/ElementMedia16.png"));
  }

  /**
   * Searches the class-path for the image with the given path 
   * and if found it returns the string representation of its URL, otherwise it returns <code>null</code>.  
   * 
   * @param path The image path to search for.
   * 
   * @return The string representation of the image URL or <code>null</code> if the image is not found.
   */
  private static String getImageURL(String path) {
    URL imageURL = DITANodeRendererCustomizer.class.getResource(path);
    if (imageURL != null) {
      return imageURL.toExternalForm();
    } else {
      logger.error(DITANodeRendererCustomizer.class.getName() + " - Image not found: " + path);
      return null;
    }
  }

  @Override
  public BasicRenderingInformation getRenderingInformation(NodeRendererCustomizerContext context) {
    BasicRenderingInformation renderingInfo = new BasicRenderingInformation();
    DitaClass ditaClass = getDITAClass(context);
    if (ditaClass != null) {
      // Get icon path from map
      String iconPath = classToIconPath.get(ditaClass);
      renderingInfo.setIconPath(iconPath);
    }
    return renderingInfo;
  }

  /**
   * Identifies the dita class of the given context.
   * 
   * @param context The context to determine the dita class for.
   * 
   * @return The dita class of the given context or <code>null</code>.
   */
  private DitaClass getDITAClass(NodeRendererCustomizerContext context) {
    DitaClass ditaClass = null;
    String classValue = context.getAttributeValue("class");
    if (classValue != null && !"".equals(classValue.trim())) {
      ditaClass = getDitaClassForClassValue(classValue);
    }
    return ditaClass;
  }

  /**
   * Identifies the DITA class for the given class value.
   * 
   * @param classValue The class attribute value.
   * @return the DITA class for the given class value.
   */
  DitaClass getDitaClassForClassValue(String classValue) {
    DitaClass[] classes = DitaClass.values();
    for (DitaClass cls : classes) {
      String value = cls.getValue();
      if (classValue.contains(value)) {
        String substring = classValue.substring(classValue.indexOf(value) + value.length() - 1);
        if (substring.trim().length() == 0 || substring.contains(value)) {
           return cls;
        } 
      }
    }
    return null;
  }
  
  @Override
  public String getDescription() {
    return "DITA Node Renderer Customizer";
  }
}