/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita.conref;

import java.io.IOException;
import java.io.StringReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.LinkedHashMap;
import java.util.StringTokenizer;

import javax.xml.transform.sax.SAXSource;

import org.apache.log4j.Logger;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.XMLReader;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.dita.DITAAccess;
import ro.sync.ecss.dita.Reference;
import ro.sync.ecss.dita.reference.keyref.KeyInfo;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.DITAConrefsResolverBase;
import ro.sync.ecss.extensions.api.ValidatingReferenceResolverException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorDocument;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.exml.workspace.api.util.XMLReaderWithGrammar;

/**
 * Resolver for content referred using <code>conref</code> attribute.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class DITAConRefResolver extends DITAConrefsResolverBase {

  /**
   * Logger for logging.
   */
  private static Logger logger = Logger.getLogger(DITAConRefResolver.class.getName());
  /**
   * The <code>keyref</code> keyref.
   */
  private static final String ATTRIBUTE_KEYREF = "keyref";
  /**
   * The <code>conref</code> attribute.
   */
  private static final String ATTRIBUTE_CONREF = "conref";
  
  /**
   * The <code>conrefend</code> attribute.
   */
  private static final String ATTRIBUTE_CONREF_END = "conrefend";
  
  /**
   * The <code>conkeyref</code> attribute.
   */
  private static final String ATTRIBUTE_CONKEYREF = "conkeyref";
  
  /**
   * Caches access to the grammar pool
   */
  private Object grammarCache = null;
  
  /**
   * <code>true</code> to resolve also keyrefs. False by default, when editing in the Author page the CSS provides the link test.
   */
  private boolean resolveAlsoKeyRefs = false;

  /**
   * An element that has <code>conref</code> attribute 
   * is an element with references.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#hasReferences(ro.sync.ecss.extensions.api.node.AuthorNode)
   */
  public boolean hasReferences(AuthorNode node) {
    boolean hasReferences = false;
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement element = (AuthorElement) node;
      AttrValue attrValue = element.getAttribute(ATTRIBUTE_CONKEYREF);
      hasReferences = attrValue != null;
      if(! hasReferences) {
        attrValue = element.getAttribute(ATTRIBUTE_CONREF);
        hasReferences = attrValue != null;
      }
      if(! hasReferences && resolveAlsoKeyRefs) {
        attrValue = element.getAttribute(ATTRIBUTE_KEYREF);
        if(attrValue != null) {
          String key = attrValue.getValue();
          if(key != null 
              //If key contains a "/", it references a subelement, so do not bother looking for metadata.
              && ! key.contains("/")) {
            LinkedHashMap<String, KeyInfo> keys = DITAAccess.getKeys(node.getXMLBaseURL());
            if(keys != null) {
              //Avoid NPE in tests 
              KeyInfo keyInfo = keys.get(key);
              if(keyInfo != null) {
                if(keyInfo.getMetaContentProvider() != null) {
                  AttrValue classValue = element.getAttribute("class");
                  if(classValue != null) {
                    hasReferences = keyInfo.getMetaContentProvider().getContent(classValue.getValue()) != null;
                  }
                }
              }
            }
          }
        }
      }
    }
    return hasReferences;
  }

  
  /**
   * Returns the value of the <code>conref</code> attribute.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#getDisplayName(ro.sync.ecss.extensions.api.node.AuthorNode)
   */
  public String getDisplayName(AuthorNode node) {
    StringBuilder displayName = new StringBuilder("conref-fragment");
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement element = (AuthorElement) node;
      AttrValue attrValue = element.getAttribute(ATTRIBUTE_CONKEYREF);
      if (attrValue != null) {
        //Conkeyref
        displayName.setLength(0);
        displayName.append(attrValue.getValue());
      } else {
        //Conref
        attrValue = element.getAttribute(ATTRIBUTE_CONREF);
        if (attrValue != null) {
          displayName.setLength(0);
          displayName.append(attrValue.getValue());
        } else {
          //Keyref
          attrValue = element.getAttribute(ATTRIBUTE_KEYREF);
          if (attrValue != null) {
            displayName.setLength(0);
            displayName.append(attrValue.getValue());
          } 
        }
      }
      //Conrefend
      attrValue = element.getAttribute(ATTRIBUTE_CONREF_END);
      if (attrValue != null) {
        displayName.insert(0, '[');
        displayName.append(" .. ").append(attrValue.getValue()).append(']');
      }
    }
    return displayName.toString();
  }
  
  /**
   * Resolve the content referred by <code>conref</code> attribute.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#resolveReference(ro.sync.ecss.extensions.api.node.AuthorNode, java.lang.String, AuthorAccess, EntityResolver)
   */
  public SAXSource resolveReference(
      AuthorNode node, 
      String systemID, 
      AuthorAccess authorAccess,
      EntityResolver  entityResolver) {
    SAXSource saxSource = null;
    
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      try {
        AuthorElement element = (AuthorElement) node;
        
        AttrValue attrValue = element.getAttribute(ATTRIBUTE_KEYREF);
        if(attrValue != null) {
          KeyInfo keyInfo = DITAAccess.getKeys(node.getXMLBaseURL()).get(attrValue.getValue());
          if(keyInfo != null) {
            if(keyInfo.getMetaContentProvider() != null) {
              AttrValue classValue = element.getAttribute("class");
              if(classValue != null) {
                String linkContent = keyInfo.getMetaContentProvider().getContent(classValue.getValue());
                  if(linkContent != null) {
                  saxSource = new SAXSource(authorAccess.getXMLUtilAccess().newNonValidatingXMLReader(), new InputSource(
                      new StringReader("<okr xmlns='http://www.oxygenxml.com/extensions/author'>" + linkContent + "</okr>")));
                }
              }
            }
          }
        } else {
          Reference conRef = null;
          boolean conKeyRef = false;
          attrValue = element.getAttribute(ATTRIBUTE_CONKEYREF);
          //Conkeyref
          if(attrValue != null) {
            String attrStringVal = attrValue.getValue();
            conRef = DITAAccess.parseDITAKeyRef(node.getXMLBaseURL(), attrStringVal);
            if(conRef == null) {
              //EXM-18993 Explanation for case in which the source was not resolved.
              throw new RuntimeException("Could not resolve the target for key reference \"" + attrStringVal + "\"");
            } else if(attrStringVal.indexOf("/") != attrStringVal.lastIndexOf("/")) {
              //Bad topic id, the key reference should have only one id.
              //EXM-18993 Explanation for case in which the source was not resolved.
              throw new RuntimeException("Format for key reference \"" + attrStringVal + "\" is invalid.");
            }
            conKeyRef = true;
          } else {
            //Conref
            attrValue = element.getAttribute(ATTRIBUTE_CONREF);
            String attrStringVal = attrValue.getValue();
            conRef = DITAAccess.parseDITAHref(systemID, attrStringVal);
          }
          if (conRef != null && conRef.getUri() != null) {
            String[] topicPath = getTopicPath(conRef.getTopicID());            
            InputSource inputSource = entityResolver.resolveEntity(null, conRef.getUri());
            if(inputSource == null) {
              inputSource = new InputSource(conRef.getUri());
            }
            
            //EXM-16083 XML Reader with grammar cache
            XMLReaderWithGrammar readerWithCache = authorAccess.getXMLUtilAccess().newNonValidatingXMLReader(grammarCache);
            XMLReader xmlReader = readerWithCache.getXmlReader();
            grammarCache = readerWithCache.getGrammarCache();
            xmlReader.setEntityResolver(entityResolver);
            
            //EXM-15209 Expand conref ranges
            String[] endRangePath = null;
            AttrValue conrefEndVal = element.getAttribute(ATTRIBUTE_CONREF_END);
            if (conrefEndVal != null) {
              String conrefEndStr = conrefEndVal.getValue();
              Reference conRefEnd = DITAAccess.parseDITAHref(systemID, conrefEndStr);
              if(conRefEnd != null && conRefEnd.getTopicID() != null && conRefEnd.getTopicID().length() > 0) {
                endRangePath = getTopicPath(conRefEnd.getTopicID());
                if(conKeyRef) {
                  //If we have "conrefend" with "conkeyref", only the last ID in the path matters.
                  if(endRangePath != null && endRangePath.length > 0) {
                    endRangePath = new String[] {endRangePath[endRangePath.length - 1]};
                  }
                }
              }
            }
            
            DITAXMLReaderWrapper readerWrapper = 
              new DITAXMLReaderWrapper(
                  xmlReader, 
                  topicPath, 
                  endRangePath, 
                  conKeyRef);
            saxSource = new SAXSource(readerWrapper, inputSource);
          }
        }
      } catch (MalformedURLException e) {
        logger.error(e, e);
      } catch (SAXNotRecognizedException e) {
        logger.error(e, e);
      } catch (SAXNotSupportedException e) {
        logger.error(e, e);
      } catch (SAXException e) {
        logger.error(e, e);
      } catch (IOException e) {
        logger.error(e, e);
      }
    }

    return saxSource;
  }
  
  /**
   * Gets the topic IDs as an array of strings.
   * 
   * @param value The <code>conref</code> attribute value.
   * @return The path of IDs as an array of strings.
   */
  protected String[] getTopicPath(String value) {
    String[] path = null;
    if(value != null) {
      //Tokenize by path separator
      StringTokenizer tokenizer = new StringTokenizer(value, "/", false);
      path = new String[tokenizer.countTokens()];
      int i = 0;
      while (tokenizer.hasMoreTokens()) {
        path[i++] = tokenizer.nextToken();
      }
    }
    return path;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  public String getDescription() {
    return "Resolves the 'conkeyref/conref/keyref' references";
  }

  /**
   * The value of <code>conref</code> attribute is used as the unique identifier.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#getReferenceUniqueID(ro.sync.ecss.extensions.api.node.AuthorNode)
   */
  public String getReferenceUniqueID(AuthorNode node) {
    String id = null;
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement element = (AuthorElement) node;
      AttrValue attrValue = element.getAttribute(ATTRIBUTE_CONKEYREF);
      //Conkeyref
      if (attrValue != null) {
        id = attrValue.getValue();
      } else {
        attrValue = element.getAttribute(ATTRIBUTE_CONREF);
        //Conref
        if (attrValue != null) {
          id = attrValue.getValue();
        } else {
          attrValue = element.getAttribute(ATTRIBUTE_KEYREF);
          //Keyref
          if (attrValue != null) {
            id = attrValue.getValue();
          }
        }
      }
    }
    return id;
  }

  /**
   * Returns <code>true</code> when the attribute name is equal to <code>'conref'</code>.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#isReferenceChanged(ro.sync.ecss.extensions.api.node.AuthorNode, java.lang.String)
   */
  public boolean isReferenceChanged(AuthorNode node, String attributeName) {
    //Check if a significant attribute changed
    return ATTRIBUTE_CONKEYREF.equals(attributeName) || ATTRIBUTE_CONREF.equals(attributeName) 
          || ATTRIBUTE_CONREF_END.equals(attributeName) || ATTRIBUTE_KEYREF.equals(attributeName);
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#getReferenceSystemID(ro.sync.ecss.extensions.api.node.AuthorNode, AuthorAccess)
   */
  public String getReferenceSystemID(AuthorNode node, AuthorAccess authorAccess) {
    String systemID = null;
    try {
      if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
        AuthorElement element = (AuthorElement) node;
        AttrValue attrValue = element.getAttribute(ATTRIBUTE_CONKEYREF);
        //Conkeyref
        if (attrValue != null) {
          String attrStringVal = attrValue.getValue();
          URL absURL = DITAAccess.resolveKeyRef(node.getXMLBaseURL(), attrStringVal, true);
          if(absURL != null) {
            systemID = absURL.toString();
          } else {
            systemID = attrStringVal;
          }
        } 
        if(systemID == null) {
          attrValue = element.getAttribute(ATTRIBUTE_CONREF);
          if (attrValue != null) {
            //Conref
            String attrStringVal = attrValue.getValue();
            systemID = new URL(node.getXMLBaseURL(), attrStringVal).toString();

            //Try to pass through catalog.
            Reference parseDITAHref = DITAAccess.parseDITAHref(
                node.getXMLBaseURL()  == null ? null : node.getXMLBaseURL().toString(), 
                    attrStringVal);
            if(parseDITAHref != null) {
              systemID = parseDITAHref.getUri();
              if(parseDITAHref.getTopicID() != null && parseDITAHref.getTopicID().length() > 0) {
                systemID +="#" + parseDITAHref.getTopicID();
              }
            }
          }
        }
        if(systemID == null) {
          attrValue = element.getAttribute(ATTRIBUTE_KEYREF);
          //keyref
          if (attrValue != null) {
            String attrStringVal = attrValue.getValue();
            KeyInfo keyInfo = DITAAccess.getKeys(node.getXMLBaseURL()).get(attrStringVal);
            if(keyInfo != null) {
              systemID = keyInfo.getDefinitionLocation().toString();
            }
          } 
        }
      }
    } catch (MalformedURLException e) {
      if (logger.isDebugEnabled()) {
        logger.debug(e, e);
      }
    }
    return systemID;
  }

  /**
   * 
   * @see ro.sync.ecss.extensions.api.ValidatingAuthorReferenceResolver#checkTarget(ro.sync.ecss.extensions.api.node.AuthorNode, ro.sync.ecss.extensions.api.node.AuthorDocument)
   */
  public void checkTarget(AuthorNode node, AuthorDocument targetDocument) throws ValidatingReferenceResolverException{
    String errorMessage = null;
    boolean sourceIsConrefPush = false;
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement sourceElement = (AuthorElement) node;
      if(sourceElement.getAttribute(ATTRIBUTE_KEYREF) != null) {
        //No validation here
        return;
      }
    }
    AuthorElement targetElement = targetDocument.getRootElement();
    if(targetElement != null) {
      if(! targetElement.getName().equals(node.getName())) {
        //Target class
        AttrValue targetClass = targetElement.getAttribute("class");
        if (targetClass == null) {
          errorMessage = "The target does not have a class attribute";
        } else {
          if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
            //Source class
            AuthorElement sourceElement = (AuthorElement) node;
            AttrValue sourceClass = sourceElement.getAttribute("class");
            if(sourceClass != null) {
              sourceIsConrefPush = sourceElement.getAttribute("conaction") != null;
              if(sourceIsConrefPush) {
                //This is actually a conref push.
                //The source must be a specialization of the target.
                boolean specialization = !DITAAccess.isGeneralizationOf(sourceClass.getValue(), targetClass.getValue());
                if (!specialization) {
                  errorMessage = "The source element \"" + sourceElement.getName() + 
                  "\" with class value \"" + sourceClass.getValue() + 
                  "\" is not a specialization of target element \"" + targetElement.getName() + 
                  "\" with class value \"" + targetClass.getValue() + "\"";
                }
              } else {
                boolean generalization = DITAAccess.isGeneralizationOf(sourceClass.getValue(), targetClass.getValue());
                if (!generalization) {
                  errorMessage = "The source element \"" + sourceElement.getName() + 
                  "\" with class value \"" + sourceClass.getValue() + 
                  "\" is not a generalization of target element \"" + targetElement.getName() + 
                  "\" with class value \"" + targetClass.getValue() + "\"";
                }
              }
            } else {
              errorMessage = "The source does not have a class attribute";
            }
          } else {
            errorMessage = "The conref source is not an element";
          }
        }
      } else {
        //Same name in target and source, probably OK
      }
    }
    if(errorMessage != null) {
      throw new ValidatingReferenceResolverException(
          (sourceIsConrefPush ? "Target of conref push was not resolved:\n" : "Conref was not expanded:\n") + errorMessage);
    }
  }


  /**
   * @see ro.sync.ecss.extensions.api.DITAConrefsResolverBase#setResolveKeyrefsToMetaContentAsConrefs(boolean)
   */
  @Override
  public void setResolveKeyrefsToMetaContentAsConrefs(boolean resolveKeyrefsAsConrefs) {
    this.resolveAlsoKeyRefs = resolveKeyrefsAsConrefs;
  }
}