/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.docbook;

import java.util.List;
import java.util.Set;

import javax.swing.text.BadLocationException;

import org.apache.log4j.Logger;

import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.ContentInterval;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.table.operations.AuthorTableDeleteColumnArguments;
import ro.sync.ecss.extensions.api.table.operations.AuthorTableDeleteRowArguments;
import ro.sync.ecss.extensions.api.table.operations.AuthorTableInsertColumnArguments;
import ro.sync.ecss.extensions.api.table.operations.AuthorTableOperationsHandler;
import ro.sync.ecss.extensions.api.table.operations.TableColumnSpecificationInformation;
import ro.sync.ecss.extensions.commons.table.operations.TableOperationsUtil;
import ro.sync.ecss.extensions.commons.table.operations.cals.CALSDocumentTypeHelper;
import ro.sync.ecss.extensions.commons.table.operations.cals.CALSTableColumnSpecificationInformation;
import ro.sync.ecss.extensions.commons.table.operations.xhtml.XHTMLDocumentTypeHelper;
import ro.sync.ecss.extensions.commons.table.support.CALSColSpec;
import ro.sync.ecss.extensions.commons.table.support.CALSTableCellInfoProvider;
import ro.sync.ecss.extensions.docbook.table.InsertTableOperation;

/**
 * Author table operations handler for Docbook framework.
 */
public class DocbookAuthorTableOperationsHandler extends AuthorTableOperationsHandler {
  /**
   * Logger for logging.
   */
  private static final Logger logger = Logger.getLogger(DocbookAuthorTableOperationsHandler.class.getName());

  /**
   * Insert CALS column operation.
   */
  private ro.sync.ecss.extensions.commons.table.operations.cals.InsertColumnOperation insertCALSColumnOperation = null;

  /**
   * Insert XHTML column operation.
   */
  private ro.sync.ecss.extensions.commons.table.operations.xhtml.InsertColumnOperation insertXHTMLColumnOperation = null;

  /**
   * Insert XHTML row operation.
   */
  private ro.sync.ecss.extensions.commons.table.operations.xhtml.InsertRowOperation insertXHTMLRowOperation = null;
  
  /**
   * Insert CALS row operation.
   */
  private ro.sync.ecss.extensions.commons.table.operations.cals.InsertRowOperation insertCALSRowOperation = null;
  
  /**
   * Insert table operation.
   */
  private InsertTableOperation insertTableOperation = null;
  
  /**
   * Delete XHTML column operation.
   */
  private ro.sync.ecss.extensions.commons.table.operations.xhtml.DeleteColumnOperation deleteXHTMLColumnOperation = null;
  
  /**
   * Delete CALS column operation. 
   */
  private ro.sync.ecss.extensions.commons.table.operations.cals.DeleteColumnOperation deleteCALSColumnOperation = null;
  
  /**
   * Delete XHTML row operation.
   */
  private ro.sync.ecss.extensions.commons.table.operations.xhtml.DeleteRowOperation deleteXHTMLRowOperation = null;
  
  /**
   * Delete CALS row operation. 
   */
  private ro.sync.ecss.extensions.commons.table.operations.cals.DeleteRowOperation deleteCALSRowOperation = null;
  
  /**
   * Docbook framework namespace.
   */
  private final String namespace;

  /**
   * Constructor.
   * 
   * @param namespace Docbook namespace. 
   */
  public DocbookAuthorTableOperationsHandler(String namespace) {
    this.namespace = namespace;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.table.operations.AuthorTableOperationsHandler#handleInsertColumn(ro.sync.ecss.extensions.api.table.operations.AuthorTableInsertColumnArguments)
   */
  @Override
  public boolean handleInsertColumn(AuthorTableInsertColumnArguments tablePasteColumnArgs) throws AuthorOperationException {
    boolean handled = false;
    try {
      AuthorAccess authorAccess = tablePasteColumnArgs.getAuthorAccess();
      AuthorNode currentNode = authorAccess.getDocumentController().getNodeAtOffset(tablePasteColumnArgs.getInsertOffset());
      boolean isXHTMLTable = false;
      
      while (currentNode != authorAccess.getDocumentController().getAuthorDocumentNode() && 
          !TableOperationsUtil.nodeHasProperties(currentNode, "table", namespace)) {
        if (TableOperationsUtil.nodeHasProperties(currentNode, "tr", namespace)) {
          isXHTMLTable = true;
          break;
        }
        currentNode = currentNode.getParent();
      }
      
      if (insertTableOperation == null) {
        insertTableOperation = new InsertTableOperation();
      }

      // Perform operation
      if (isXHTMLTable) {
        if (insertXHTMLRowOperation == null) {
          insertXHTMLRowOperation = new ro.sync.ecss.extensions.commons.table.operations.xhtml.InsertRowOperation();
        }
        if (insertXHTMLColumnOperation == null) {
          insertXHTMLColumnOperation = new ro.sync.ecss.extensions.commons.table.operations.xhtml.InsertColumnOperation();
        }
        insertXHTMLColumnOperation.performInsertColumn(
            authorAccess, 
            namespace, 
            tablePasteColumnArgs.getColumnFragments(), 
            tablePasteColumnArgs.getColumnSpecificationInformation(),
            tablePasteColumnArgs.areFragmentsWrappedInCells(), 
            insertXHTMLRowOperation,
            insertTableOperation);
      } else {
        if (insertCALSRowOperation == null) {
          insertCALSRowOperation = new ro.sync.ecss.extensions.commons.table.operations.cals.InsertRowOperation();
        }
        if (insertCALSColumnOperation == null) {
          insertCALSColumnOperation = new ro.sync.ecss.extensions.commons.table.operations.cals.InsertColumnOperation();
        }
        insertCALSColumnOperation.performInsertColumn(
            authorAccess, 
            namespace, 
            tablePasteColumnArgs.getColumnFragments(), 
            tablePasteColumnArgs.getColumnSpecificationInformation(),
            tablePasteColumnArgs.areFragmentsWrappedInCells(), 
            insertCALSRowOperation, 
            insertTableOperation);
      }
      // The paste operation was handled
      handled = true;
    } catch (IllegalArgumentException e) {
        logger.warn("Paste column failed.", e);
    } catch (BadLocationException e) {
        logger.warn("Paste column failed.", e);
    }
    return handled;
  }

  /**
   * @see ro.sync.ecss.extensions.api.table.operations.AuthorTableOperationsHandler#handleDeleteColumn(ro.sync.ecss.extensions.api.table.operations.AuthorTableDeleteColumnArguments)
   */
  @Override
  public boolean handleDeleteColumn(AuthorTableDeleteColumnArguments arguments)
  throws AuthorOperationException {
    boolean handled = false;

    AuthorAccess authorAccess = arguments.getAuthorAccess();
    
    AuthorNode currentNode = null;
    try {
      List<ContentInterval> intervals = arguments.getColumnCellsIntervals();
      if (intervals != null && intervals.size() > 0) {
        int firstCellOffset = intervals.get(0).getEndOffset();
        currentNode = authorAccess.getDocumentController().getNodeAtOffset(firstCellOffset);
      }
      if (currentNode != null) {
        boolean isXHTMLTable = false;
        // Determine the table type
        while (currentNode != authorAccess.getDocumentController().getAuthorDocumentNode() && 
            !TableOperationsUtil.nodeHasProperties(currentNode, "table", namespace)) {
          if (TableOperationsUtil.nodeHasProperties(currentNode, "tr", namespace)) {
            isXHTMLTable = true;
            break;
          }
          currentNode = currentNode.getParent();
        }
        if (isXHTMLTable) {
          // Invoke delete column for XHTML table
          if (deleteXHTMLColumnOperation == null) {
            deleteXHTMLColumnOperation = new ro.sync.ecss.extensions.commons.table.operations.xhtml.DeleteColumnOperation();
          }
          handled = deleteXHTMLColumnOperation.performDeleteColumn(authorAccess, arguments.getColumnCellsIntervals(), false);
        } else {
          // Invoke delete column for CALS table
          if (deleteCALSColumnOperation == null) {
            deleteCALSColumnOperation = new ro.sync.ecss.extensions.commons.table.operations.cals.DeleteColumnOperation();
          }
          handled = deleteCALSColumnOperation.performDeleteColumn(authorAccess, arguments.getColumnCellsIntervals(), false);
        }
      }
    } catch (BadLocationException e) {
      // Do nothing
      logger.warn("Delete selected column failed", e);
    }
    
    return handled;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.table.operations.AuthorTableOperationsHandler#handleDeleteRow(ro.sync.ecss.extensions.api.table.operations.AuthorTableDeleteRowArguments)
   */
  @Override
  public boolean handleDeleteRow(AuthorTableDeleteRowArguments arguments)
  throws AuthorOperationException {
    boolean handled = false;
    
    AuthorAccess authorAccess = arguments.getAuthorAccess();
    ContentInterval rowInterval = arguments.getRowInterval();
    try {
      AuthorNode rowNode = authorAccess.getDocumentController().getNodeAtOffset(rowInterval.getStartOffset() + 1);
      if (rowNode.getStartOffset() == rowInterval.getStartOffset() && rowNode.getEndOffset() == rowInterval.getEndOffset() - 1) {
        boolean isXHTMLTable = false;
        // Determine the table type
        if (TableOperationsUtil.nodeHasProperties(rowNode, "tr", namespace)) {
          isXHTMLTable = true;
        }

        if (isXHTMLTable) {
          // Invoke delete column for XHTML table
          if (deleteXHTMLRowOperation == null) {
            deleteXHTMLRowOperation = new ro.sync.ecss.extensions.commons.table.operations.xhtml.DeleteRowOperation();
          }
          handled = deleteXHTMLRowOperation.performDeleteRows(authorAccess, rowInterval.getStartOffset(), rowInterval.getEndOffset());
        } else {
          // Invoke delete column for CALS table
          if (deleteCALSRowOperation == null) {
            deleteCALSRowOperation = new ro.sync.ecss.extensions.commons.table.operations.cals.DeleteRowOperation();
          }
          handled = deleteCALSRowOperation.performDeleteRows(authorAccess, rowInterval.getStartOffset(), rowInterval.getEndOffset());
        }
      } else {
        throw new AuthorOperationException(
            "The operation cannot be performed due to: wrong row content interval argument");
      }
    } catch (BadLocationException e) {
      throw new AuthorOperationException(
          "The operation cannot be performed due to: " + e.getMessage(), e);
    }

    return handled;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.table.operations.AuthorTableOperationsHandler#getTableElementContainingOffset(ro.sync.ecss.extensions.api.AuthorAccess, int)
   */
  @Override
  public AuthorElement getTableElementContainingOffset(AuthorAccess access, int offset) {
    String[] xhtmlTableElementNames = XHTMLDocumentTypeHelper.TABLE_ELEMENT_NAMES;
    // Create the Docbook table element names list
    String[] tableElementNames = new String[xhtmlTableElementNames.length + 1];
    for (int i = 0; i < xhtmlTableElementNames.length; i++) {
      tableElementNames[i] = xhtmlTableElementNames[i];
    }
    tableElementNames[tableElementNames.length - 1] = CALSDocumentTypeHelper.ELEMENT_NAME_TABLE;
   
    return TableOperationsUtil.getTableElementContainingOffset(offset, namespace, 
        access, tableElementNames);
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.table.operations.AuthorTableOperationsHandler#getColumnSpecification(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.node.AuthorElement, int)
   */
  @Override
  public TableColumnSpecificationInformation getColumnSpecification(AuthorAccess access,
      AuthorElement tableElement, int columnIndex) {
    TableColumnSpecificationInformation colspecInfo = null;
    if (tableElement != null) {
      // CALS table
      CALSTableCellInfoProvider cellInfoProvider = new CALSTableCellInfoProvider();
      cellInfoProvider.init(tableElement);
      // Determine the column specification 
      Set<CALSColSpec> colSpecs = cellInfoProvider.getColSpecs();
      if (colSpecs.size() > columnIndex) {
        // Found the column specification
        CALSColSpec colSpec = colSpecs.toArray(new CALSColSpec[0])[columnIndex];
        if (colSpec != null) {
          colspecInfo = new CALSTableColumnSpecificationInformation(
              colSpec.getColWidth(), colSpec.getColumnName());
        }
      }
    }
    return colspecInfo;
  }
  
}
