/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.docbook.link;

import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.WeakHashMap;

import javax.swing.text.BadLocationException;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorListenerAdapter;
import ro.sync.ecss.extensions.api.link.InvalidLinkException;
import ro.sync.ecss.extensions.api.link.LinkTextResolver;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorDocument;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.node.AuthorParentNode;
import ro.sync.xml.XmlUtil;

/**
 * Resolves local docbook xrefs. The content of the link is given by either
 * the xreflabel attribute or a title(info/title) child of the targeted element.  
 * 
 * @since 14.2
 * <br>
 * <br>
 * *********************************
 * <br>
 * EXPERIMENTAL - Subject to change
 * <br>
 * ********************************
 * </br>
 * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
 * comments about it, please let us know.</p>
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public class DocbookLinkTextResolver extends LinkTextResolver {
  /**
   * Logger for logging.
   */
  private static final Logger logger = Logger.getLogger(DocbookLinkTextResolver.class.getName());

  /**
   * Keeps a mapping between all the evaluated xref elements and the name of their 
   * targeted element. It uses weak references to avoid a memory leak.
   */
  private WeakHashMap<AuthorNode, String> dependencies = new WeakHashMap<AuthorNode, String>();
  /**
   * Decides when a certain xref element needs to be refreshed because the targeted
   * element has changed.
   */
  private AuthorListenerAdapter authorListenerAdapter = new AuthorListenerAdapter() {
    public void attributeChanged(ro.sync.ecss.extensions.api.AttributeChangedEvent e) {
      Set<AuthorNode> nodes = dependencies.keySet();
      for (AuthorNode authorNode : nodes) {
        String elementName = dependencies.get(authorNode);
        if (elementName == null || elementName.equals(e.getOwnerAuthorNode().getDisplayName())) {
          // Either this XREF wasn't matched to a target or the target has changed.
          authorAccess.getEditorAccess().refresh(authorNode);
        }
      }
    }
  };


  private AuthorAccess authorAccess;
  
  /**
   * @see ro.sync.ecss.extensions.api.link.LinkTextResolver#resolveReference(ro.sync.ecss.extensions.api.node.AuthorNode)
   */
  @Override
  public String resolveReference(AuthorNode node) throws InvalidLinkException {
    if (logger.isDebugEnabled()) {
      logger.debug("Resolve " + node);
    }
    String linkText = null;
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT 
        && "xref".equals(XmlUtil.getLocalName(node.getName()))) {
      AuthorElement element = (AuthorElement) node;
      AttrValue linkendAttr = element.getAttribute("linkend");
      if (linkendAttr != null && linkendAttr.getValue() != null) {
        String elementName = null;
        AuthorDocument ownerDocument = node.getOwnerDocument();
        if (ownerDocument != null) {
          AuthorElement referredElement = findElementsWithUniqueIds(ownerDocument, linkendAttr.getValue());
          if (logger.isDebugEnabled()) {
            logger.debug("Referred " + referredElement);
          }
          if (referredElement != null) {
            AttrValue xrefLabelAttr = referredElement.getAttribute("xreflabel");
            if (xrefLabelAttr != null && xrefLabelAttr.getValue() != null) {
              linkText = xrefLabelAttr.getValue();
            } else {
              linkText = getTitleValue(referredElement);
            }
            
            elementName = referredElement.getDisplayName();
          } else {
            // We could throw an exception but this could be a module and since
            // we don't have a "master" file context we choose to fail silently.
          }
        }

        if (logger.isDebugEnabled()) {
          logger.debug(hashCode() + " put " + node + " to " + elementName);
        }
        dependencies.put(node, elementName);
      }
    }
    
    return linkText;
  }

  /**
   * @see ro.sync.ecss.extensions.api.link.LinkTextResolver#refresh()
   */
  @Override
  public void refresh() {
    dependencies.clear();
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.link.LinkTextResolver#clearReferencesCache()
   */
  @Override
  public void clearReferencesCache() {
    dependencies.clear();
  }
  
  /**
   * Gather all elements that have id attributes.
   * 
   * @param node The current node.
   * @param id The searched ID.
   * 
   * @return The node with the given ID. 
   */
  private static AuthorElement findElementsWithUniqueIds(AuthorNode node, String id) {
    AuthorElement searched = null;
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement elem = (AuthorElement) node;
      if (getIDAttribute(elem, id) != null) {
        searched = elem;
      }
    }
    
     if (searched == null) {
      List<AuthorNode> childNodes = ((AuthorParentNode) node).getContentNodes();
      for (int i = 0; i < childNodes.size(); i++) { 
        searched = findElementsWithUniqueIds(childNodes.get(i), id);
        if (searched != null) {
          break;
        }
      }
    }
    
    return searched;
  }
  
  /**
   * Return the value of the ID attribute.
   * 
   * @param elem The current element.
   * @param id The searched id.
   * 
   * @return The attribute name.
   */
  private static String getIDAttribute(AuthorElement elem, String id) {
    String found = null;
    int count = elem.getAttributesCount();
    for (int i = 0; i < count; i++) {
      String attributeName = elem.getAttributeAtIndex(i);
      if ("id".equals(attributeName) || "xml:id".equals(attributeName)) {
        String idValue = elem.getAttribute(attributeName).getValue();
        if (id.equals(idValue)) {
          found = attributeName;
          break;
        }
      }
    }
    
    return found;
  }
  
  /**
   * Checks if the element has a TITLE child or an INFO/TITLE child and returns 
   * it's value.
   * 
   * @param elem The current element.
   * @return The title value.
   */
  public static String getTitleValue(AuthorElement elem) {
    String title = null;
    try {
      AuthorElement firstElement = getTitleChild(elem);
      if (firstElement != null) {
        // We have a candidate.
        if ("title".equals(firstElement.getLocalName())) {
          title = firstElement.getTextContent();
        } else if ("info".equals(firstElement.getDisplayName())) {
          // We should find a title in it.
          AuthorNode titleElement = getTitleChild(firstElement);

          if (titleElement != null && "title".equals(titleElement.getDisplayName())) {
            title = titleElement.getTextContent();  
          }
        }
      }
    } catch (BadLocationException e) {}

    return title;
  }

  /**
   * Find the title child of this element.
   * 
   * @param elem Author element.
   * 
   * @return The DocBook title element.
   */
  private static AuthorElement getTitleChild(AuthorElement elem) {
    List<AuthorNode> contentNodes = elem.getContentNodes();
    AuthorElement firstElement = null;
    for (Iterator iterator = contentNodes.iterator(); iterator.hasNext();) {
      AuthorNode authorNode = (AuthorNode) iterator.next();
      if (authorNode.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
        firstElement = (AuthorElement) authorNode;
        break;
      }
    }
    return firstElement;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorExtensionStateListener#activated(ro.sync.ecss.extensions.api.AuthorAccess)
   */
  @Override
  public void activated(AuthorAccess authorAccess) {
    this.authorAccess = authorAccess;
    authorAccess.getDocumentController().addAuthorListener(authorListenerAdapter);
  }

  /**
   * @see ro.sync.ecss.extensions.api.AuthorExtensionStateListener#deactivated(ro.sync.ecss.extensions.api.AuthorAccess)
   */
  @Override
  public void deactivated(AuthorAccess authorAccess) {
    this.authorAccess = null;
    authorAccess.getDocumentController().removeAuthorListener(authorListenerAdapter);
  }
}
