/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.docbook.link;

import javax.swing.text.BadLocationException;

import org.apache.log4j.Logger;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.commons.ExtensionTags;
import ro.sync.ecss.extensions.commons.operations.CommonsOperationsUtil;

/**
 * Insert Web Link (link).
 * 
 * @author adriana_sbircea
 * @author costi
 */
public class InsertExternalLinkOperation implements AuthorOperation {
  /**
   * Logger for logging. 
   */
  protected static final Logger logger = Logger.getLogger(InsertExternalLinkOperation.class.getName());
  
  /**
   * Docbook namespace.
   */
  private static final String DOCBOOK_NS = "http://docbook.org/ns/docbook";
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
   */
  @Override
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
      throws IllegalArgumentException, AuthorOperationException {
    // Try to insert a link element with href attribute
    insertExternalLink(authorAccess, true);
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "Insert an external link";
  }

  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
   */
  @Override
  public ArgumentDescriptor[] getArguments() {
    return null;
  }
  
  /**
   * Insert a web link.
   * 
   * @param authorAccess  The author access.
   * @param schemaAware     The schema aware.
   * 
   * @throws AuthorOperationException 
   */
  public void insertExternalLink(AuthorAccess authorAccess, boolean schemaAware) throws AuthorOperationException {
    // Get the chosen url
    String url = InsertLinkUtil.chooseURLForLink(
        authorAccess, authorAccess.getAuthorResourceBundle().getMessage(ExtensionTags.INSERT_WEB_LINK));
    if (url != null) {
      StringBuilder linkXMLFragment = new StringBuilder();

      // Link element
      linkXMLFragment.append("<link ");
      
      int caretOffset = authorAccess.getEditorAccess().getCaretOffset();

      // Determine if it is Docbook 5.
      // Go up to find an element.        
      boolean addNSDecl = false;
      AuthorNode node = null;
      try {
        node = authorAccess.getDocumentController().getNodeAtOffset(caretOffset);
      } catch (BadLocationException e) {
        logger.warn(e, e);
      }          
      while (node != null && node.getType() != AuthorNode.NODE_TYPE_ELEMENT ){
        node = node.getParent();
      }
      if(node != null) {
        if(DOCBOOK_NS.equals(((AuthorElement)node).getNamespace())){
          // It is Docbook 5.
          addNSDecl = true;
        }
      }
      
      if (addNSDecl) {
        // Add the namespace
        linkXMLFragment.append("xmlns='").append(DOCBOOK_NS).append("' xmlns:xlink=\"http://www.w3.org/1999/xlink\" ");

        // Add the xlink:href attribute.
        linkXMLFragment.append("xlink:href=\"").append(url).append("\"");
      } else {
        // Add the href attribute.
        linkXMLFragment.append("href=\"").append(url).append("\"");
      }
      
      // Close the link element
      linkXMLFragment.append("/>");
      
      // If there is a selection, surround the selection with the given fragment else 
      // try to insert the fragment at caret
      CommonsOperationsUtil.surroundWithFragment(authorAccess, schemaAware, linkXMLFragment.toString());
    } else {
      // User canceled the operation 
    }
  }
}