/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2011 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.xhtml;

import java.util.ArrayList;
import java.util.List;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.contentcompletion.xml.CIAttribute;
import ro.sync.contentcompletion.xml.CIElement;
import ro.sync.contentcompletion.xml.CIValue;
import ro.sync.contentcompletion.xml.Context;
import ro.sync.contentcompletion.xml.ContextElement;
import ro.sync.contentcompletion.xml.SchemaManagerFilter;
import ro.sync.contentcompletion.xml.WhatAttributesCanGoHereContext;
import ro.sync.contentcompletion.xml.WhatElementsCanGoHereContext;
import ro.sync.contentcompletion.xml.WhatPossibleValuesHasAttributeContext;

/**
 * XHTML implementation for schema manager filter for adding type attribute values
 * for script and style elements in content completion proposals list.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class XHTMLSchemaManagerFilter implements SchemaManagerFilter {
  /**
   * <code>script</code> element types. 
   */
  private String[] scriptTypes = new String[] {
    "text/javascript", 
    "application/javascript",
    "text/ecmascript", 
    "application/ecmascript", 
    "application/json"
  };

  /**
   * <code>style</code> element types
   */
  private String[] styleTypes = new String[] { "text/css" };
  
  /**
   * @see ro.sync.contentcompletion.xml.SchemaManagerFilter#filterElements(java.util.List, ro.sync.contentcompletion.xml.WhatElementsCanGoHereContext)
   */
  @Override
  public List<CIElement> filterElements(List<CIElement> elements, WhatElementsCanGoHereContext context) {
    // No filter
    return elements;
  }

  /**
   * @see ro.sync.contentcompletion.xml.SchemaManagerFilter#filterAttributes(java.util.List, ro.sync.contentcompletion.xml.WhatAttributesCanGoHereContext)
   */
  @Override
  public List<CIAttribute> filterAttributes(
      List<CIAttribute> attributes, WhatAttributesCanGoHereContext context) {
    // No filter
    return attributes;
  }

  /**
   * @see ro.sync.contentcompletion.xml.SchemaManagerFilter#filterAttributeValues(java.util.List, ro.sync.contentcompletion.xml.WhatPossibleValuesHasAttributeContext)
   */
  @Override
  public List<CIValue> filterAttributeValues(
      List<CIValue> attributeValues, WhatPossibleValuesHasAttributeContext context) {
    if (context != null) {
      ContextElement parentElement = context.getParentElement();
      if (parentElement != null && "type".equals(context.getAttributeName())) {
        // Filter values for style type attribute
        String qName = parentElement.getQName();
        String ns = parentElement.getNamespace();
        if ("http://www.w3.org/1999/xhtml".equals(ns)) {
          String localName = getLocalName(qName);
          if ("style".equals(localName)) {
            if (attributeValues == null) {
              attributeValues = new ArrayList<CIValue>();
            }
            for (String styleType : styleTypes) {
              // Add style types
              attributeValues.add(new CIValue(styleType));
            }
          } else if ("script".equals(localName)) {
            // Filter values for script type attribute
            if (attributeValues == null) {
              attributeValues = new ArrayList<CIValue>();
            }
            for (String scriptType : scriptTypes) {
              // Add script types
              attributeValues.add(new CIValue(scriptType));
            }
          }
        }
      }
    }
   
    return attributeValues;
  }
  
  /**
   * Get the local name from an qualified element or attribute name.
   * 
   * @param qName Qualified name.
   * @return the local name, or null if the argument is null.
   */
  public static String getLocalName(String qName) {
    String local = qName;
    if (qName != null) {
      int idx = qName.lastIndexOf(':');
      if (idx != -1) {
        local = qName.substring(idx + 1);
      }
    }
    return local;
  }

  /**
   * @see ro.sync.contentcompletion.xml.SchemaManagerFilter#filterElementValues(java.util.List, ro.sync.contentcompletion.xml.Context)
   */
  @Override
  public List<CIValue> filterElementValues(List<CIValue> elementValues, Context context) {
    // No filter
    return elementValues;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "XHTML implementation for adding type attribute values " + 
    		"for script and style elements in content completion proposals list."; 
  }
}