/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.xsd;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.exml.workspace.api.node.customizer.BasicRenderingInformation;
import ro.sync.exml.workspace.api.node.customizer.NodeRendererCustomizerContext;
import ro.sync.exml.workspace.api.node.customizer.XMLNodeRendererCustomizer;

/**
 * Class used to customize the way an XML Schema node is rendered in the UI. 
 * A node represents an entry from Author outline, Author bread crumb,
 * Text page outline, content completion proposals window or Elements view.
 * 
 * @author alina_iordache
 */
@API(type = APIType.INTERNAL, src = SourceType.PUBLIC)
public class XSDNodeRendererCustomizer extends XMLNodeRendererCustomizer {
  /**
   * Logger for logging.
   */
  private static final Logger logger = Logger.getLogger(XSDNodeRendererCustomizer.class.getName());
  /**
   * Mapping between name and icon path
   */
  private static final Map<String, String> nameToIconPath = new HashMap<String, String>();

  // Initialize mapping
  static {
    // assert
    nameToIconPath.put("assert", getImageURL("/images/node-customizer/Assert16.png"));

    // alternative
    nameToIconPath.put("alternative", getImageURL("/images/node-customizer/Alternative16.png"));
    
    // attribute
    nameToIconPath.put("attribute", getImageURL("/images/node-customizer/AttributeDeclaration16.png"));

    // attribute group
    nameToIconPath.put("attributeGroup", getImageURL("/images/node-customizer/AttributeGroup16.gif"));

    // Element
    nameToIconPath.put("element", getImageURL("/images/node-customizer/ElementDeclaration16.gif"));

    // element group
    nameToIconPath.put("group", getImageURL("/images/node-customizer/ModelGroup16.gif"));

    // notation
    nameToIconPath.put("notation", getImageURL("/images/node-customizer/Notation16.gif"));

    // complex type
    nameToIconPath.put("complexType", getImageURL("/images/node-customizer/ComplexType16.gif"));

    // simple type
    nameToIconPath.put("simpleType", getImageURL("/images/node-customizer/SimpleType16.gif"));

    // import
    nameToIconPath.put("import", getImageURL("/images/node-customizer/Import16.png"));

    // include
    nameToIconPath.put("include", getImageURL("/images/node-customizer/Include16.png"));

    // redefine
    nameToIconPath.put("redefine", getImageURL("/images/node-customizer/Redefine16.png"));

    // all
    nameToIconPath.put("all", getImageURL("/images/node-customizer/ElementAll16.png"));

    // choice
    nameToIconPath.put("choice", getImageURL("/images/node-customizer/ElementChoice16.png"));

    // sequence
    nameToIconPath.put("sequence", getImageURL("/images/node-customizer/ElementSequence16.png"));

    // schema
    nameToIconPath.put("schema", getImageURL("/images/node-customizer/ElementSchema16.png"));
    
    // openContent
    nameToIconPath.put("openContent", getImageURL("/images/node-customizer/ElementOpenContent16.png"));
    
    // defaultOpenContent
    nameToIconPath.put("defaultOpenContent", getImageURL("/images/node-customizer/ElementOpenContent16.png"));

    // override
    nameToIconPath.put("override", getImageURL("/images/node-customizer/ElementOverride16.png"));
  }

  /**
   * Searches the class-path for the image with the given path 
   * and if found it returns the string representation of its URL, otherwise it returns <code>null</code>.  
   * 
   * @param path The image path to search for.
   * 
   * @return The string representation of the image URL or <code>null</code> if the image is not found.
   */
  private static String getImageURL(String path) {
    URL imageURL = XSDNodeRendererCustomizer.class.getResource(path);
    if (imageURL != null) {
      return imageURL.toExternalForm();
    } else {
      logger.error(XSDNodeRendererCustomizer.class.getName() + " - Image not found: " + path);
      return null;
    }
  }
  
  /**
   * @see ro.sync.exml.workspace.api.node.customizer.XMLNodeRendererCustomizer#getRenderingInformation(ro.sync.exml.workspace.api.node.customizer.NodeRendererCustomizerContext)
   */
  @Override
  public BasicRenderingInformation getRenderingInformation(NodeRendererCustomizerContext context) {
    BasicRenderingInformation renderingInfo = new BasicRenderingInformation();
    String nodeName = context.getNodeName();
    if (nodeName != null) {
      String nodeNS = context.getNodeNamespace();
      String iconPath = null;
      if ("http://www.w3.org/2001/XMLSchema".equals(nodeNS)) {
        String nameWithoutPrefix = 
            nodeName.indexOf(":") != -1 ? nodeName.substring(nodeName.indexOf(":") + 1) : nodeName;
        // Get icon path from map.
        iconPath = nameToIconPath.get(nameWithoutPrefix);
        renderingInfo.setIconPath(iconPath);
      } 
    }
    return renderingInfo;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "XML Schema Node Renderer Customizer";
  }
}