/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.structure;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.exml.workspace.api.node.customizer.BasicRenderingInformation;

/**
 * The rendering information used to render a node in the outliner and bread crumb.
 * 
 * @since 11.2
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public class RenderingInformation extends BasicRenderingInformation {

  /**
   * The node to render
   */
  private AuthorNode node;
  
  /**
   * The rendered text. This will be used only in the Outliner.
   * By default it shows some node text content. 
   */
  private String additionalRenderedText;

  /**
   * The additional rendered attribute value. This will be used only in the Outliner.
   * By default it shows the value of the first attribute. 
   */
  private String additionalRenderedAttributeValue;
  
  /**
   * Set this from the user side to ignore the node from display
   */
  private boolean ignoreNodeFromDisplay;
  
  /**
   * @param node The node to render
   * @param renderedText  The rendered text. This will be used both in the Outliner and the Bread Crumb.
   * By default it is usually the node name. 
   * @param additionalRenderedText The additional rendered text. This will be used only in the Outliner.
   * By default it shows some node text content. 
   * @param tooltipText The tooltip text which will appear in the tooltip associated with the node
   */
  public RenderingInformation(AuthorNode node, String renderedText, String additionalRenderedText,
      String tooltipText) {
    this(node, renderedText, additionalRenderedText, null, tooltipText);
  }
  
  /**
   * @param node The node to render
   * @param renderedText  The rendered text. This will be used both in the Outliner and the Bread Crumb.
   * By default it is usually the node name. 
   * @param additionalRenderedText The additional rendered text. This will be used only in the Outliner.
   * By default it shows some node text content . 
   * @param additionalRenderedAttributeValue The additional rendered attribute value. This will be used only in the Outliner.
   * By default it shows the value of the first attribute. 
   * @param tooltipText The tooltip text which will appear in the tooltip associated with the node
   */
  public RenderingInformation(AuthorNode node, String renderedText, String additionalRenderedText, String additionalRenderedAttributeValue,
      String tooltipText) {
    this.node = node;
    setRenderedText(renderedText);
    this.additionalRenderedText = additionalRenderedText;
    this.additionalRenderedAttributeValue = additionalRenderedAttributeValue;
    setTooltipText(tooltipText);
  }
  
  /**
   * The additional rendered text. This will be used only in the Outliner.
   * By default it shows some node text content. 
   * 
   * @param additionalRenderedText The additional rendered text. This will be used only in the Outliner.
   * By default it shows some node text content. 
   */
  public void setAdditionalRenderedText(String additionalRenderedText) {
    this.additionalRenderedText = additionalRenderedText;
  }
  
  /**
   * Set the additional rendered attribute value. This will be used only in the Outliner.
   * By default it shows the value of the first attribute. 
   * 
   * @param additionalRenderedAttributeValue The additional rendered attribute value.
   */
  public void setAdditionalRenderedAttributeValue(String additionalRenderedAttributeValue) {
    this.additionalRenderedAttributeValue = additionalRenderedAttributeValue;
  }
  
  /**
   * The additional rendered text. This will be used only in the Outliner.
   * By default it shows some node text content. 
   * 
   * @return Returns  the additional rendered text. This will be used only in the Outliner.
   * By default it shows the value of the first attribute and some text. 
   */
  public String getAdditionalRenderedText() {
    return additionalRenderedText;
  }
  
  /**
   * Get the additional rendered attribute value. This will be used only in the Outliner.
   * By default it shows the value of the first attribute.
   * 
   * @return Returns the the additional rendered attribute value. This will be used only in the Outliner.
   * By default it shows the value of the first attribute.
   */
  public String getAdditionalRenderedAttributeValue() {
    return additionalRenderedAttributeValue;
  }
  
  /**
   * @return Returns the node to render information for.
   */
  public AuthorNode getNode() {
    return node;
  }
  
  /**
   * Set this to <code>true</code> to ignore this node from being displayed.
   * This takes effect only on the Breadcrumb Customizer.
   * 
   * @param ignoreNodeFromDisplay Set this to <code>true</code> to ignore this node from being displayed.
   * 
   * @since 12.1
   */
  public void setIgnoreNodeFromDisplay(boolean ignoreNodeFromDisplay) {
    this.ignoreNodeFromDisplay = ignoreNodeFromDisplay;
  }
  
  /**
   * Check if this node should be ignored for display, used only on the breadcrumb.
   * 
   * @return Returns <code>true</code> to ignore this node from being displayed.
   */
  public boolean isIgnoreNodeFromDisplay() {
    return ignoreNodeFromDisplay;
  }
}
